"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@phylum/command");
const path_1 = require("path");
const read = require("read");
const colors = require("ansi-colors");
const cli_args_1 = require("../cli-args");
const fs_extra_1 = require("fs-extra");
const child_process_1 = require("child_process");
async function run(command, argv) {
    const args = new command_1.CommandSpec([
        { name: "path", defaultFallback: true, defaultValue: "." },
        cli_args_1.PACKAGE_NAME,
        cli_args_1.PACKAGE_TYPE
    ]).parse(argv);
    const path = path_1.resolve(args.path);
    const name = args.name || (await question("Enter a package name", args.name || path_1.basename(path), name => {
        if (cli_args_1.isValidPackageName(name)) {
            return true;
        }
        console.log(`"${name}" is not a valid package name.`);
    }));
    const type = args.type || (await question("Do you want to create an application (a) or a library (L)?", "a", type => {
        if (["a", "l"].includes(type.toLowerCase())) {
            return true;
        }
        console.log(`Please enter "a" for application or "L" for library.`);
    }, type => {
        return { a: "application", l: "library" }[type.toLowerCase()];
    }));
    await fs_extra_1.ensureDir(path);
    const contents = (await fs_extra_1.readdir(path)).filter(n => /^(?:src|package.json|vuechain.json|tsconfig.json)$/.test(n));
    if (contents.length > 0) {
        console.log(colors.redBright(`The directory already contains: ${contents.join(", ")}`));
        process.exitCode = 1;
        return;
    }
    console.log(`Creating new vuechain ${colors.blueBright(type)} "${colors.greenBright(name)}" ${colors.gray(`(in: ${path})`)}`);
    await fs_extra_1.writeFile(path_1.join(path, "package.json"), JSON.stringify({
        name,
        version: "1.0.0",
        description: `A vuechain ${type}`,
        main: type === "library" ? "./dist" : undefined,
        scripts: {
            start: "vuechain start",
            build: "vuechain build",
            clean: "vuechain clean",
            prepack: "vuechain clean && vuechain build"
        },
        dependencies: {},
        devDependencies: {}
    }, null, "\t") + "\n");
    await fs_extra_1.writeFile(path_1.join(path, "vuechain.json"), JSON.stringify({
        packageType: type,
        prefix: type === "library" ? name : undefined
    }, null, "\t") + "\n");
    await fs_extra_1.copy(path_1.join(__dirname, "../../prefabs", type), path);
    await new Promise((resolve, reject) => {
        const npm = child_process_1.spawn("npm", ["install", "--save-dev", "vuechain"], {
            cwd: path,
            stdio: "inherit",
            shell: true
        });
        npm.on("error", reject);
        npm.on("exit", (code, signal) => {
            if (code || signal) {
                reject(code || signal);
            }
            else {
                resolve();
            }
        });
    });
}
exports.run = run;
async function question(prompt, defaultValue, validate, map) {
    let value;
    while (!value || (validate && !validate(value))) {
        value = await new Promise((resolve, reject) => {
            read({
                prompt: `${prompt}${defaultValue ? ` (${defaultValue})` : ""}> `
            }, (error, value) => {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(value.trim() || defaultValue);
                }
            });
        });
    }
    return (map ? map(value) : value);
}
//# sourceMappingURL=init.js.map