"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const package_manifest_1 = require("../../i18n/package-manifest");
const parse_source_1 = require("../../i18n/parse-source");
const config_1 = require("../../config");
const justify_source_1 = require("../../i18n/justify-source");
const common_1 = require("./common");
const messages_1 = require("../../i18n/messages");
const path_1 = require("path");
const NAME = "vuechain i18n plugin";
class I18nPlugin {
    constructor(config, runner, adapter) {
        this.config = config;
        this.runner = runner;
        this.adapter = adapter;
        this._packageConfigRequests = new Map();
        this._packageManifestRequests = new Map();
        this._sourceRequests = new Map();
        this._processingModules = new Map();
        this._dependencies = new Map();
        this._locales = new Map();
        this.files = new Map();
    }
    /** @internal */
    apply(compiler) {
        compiler.hooks.compilation.tap(NAME, (compilation) => {
            compilation[common_1.I18N_PLUGIN_KEY] = this;
            if (!compilation.compiler.parentCompilation) {
                this._packageConfigRequests.clear();
                this._packageManifestRequests.clear();
                this._sourceRequests.clear();
                this._processingModules.clear();
                this._locales.clear();
                this.files.clear();
            }
            compilation.hooks.buildModule.tap(NAME, (module) => {
                this._sourceRequests.delete(module.resource);
                this._processingModules.delete(module.resource);
            });
            compilation.hooks.succeedModule.tap(NAME, (module) => {
                this.processModule(compiler, module);
            });
            compilation.hooks.finishModules.tapPromise(NAME, async (modules) => {
                for (const module of modules) {
                    this.processModule(compiler, module);
                }
                await Promise.all(this._processingModules.values());
            });
        });
        compiler.hooks.emit.tapPromise(NAME, async (compilation) => {
            if (!compilation.getStats().hasErrors()) {
                if (this.adapter) {
                    await this.adapter.process(this);
                }
                for (const [, dependency] of this._dependencies) {
                    const manifest = await this.getPackageManifest(dependency.config);
                    if (manifest) {
                        for (const locale of manifest.locales) {
                            const assetPath = path_1.join(dependency.config.outDir, messages_1.getLocaleMessageAssetPath(locale));
                            const messages = await fs_extra_1.readJson(assetPath);
                            this.bundleMessages(locale, messages);
                            // TODO: Only add messages that are required by the dependency.
                        }
                    }
                }
                for (const [locale, messages] of this._locales) {
                    const asset = Buffer.from(this.runner.env === "production"
                        ? JSON.stringify(messages)
                        : JSON.stringify(messages, null, "\t"));
                    const assetPath = messages_1.getLocaleMessageAssetPath(locale);
                    if (compilation.assets[assetPath]) {
                        throw new Error(`Unable to add locale messages to webpack output. There is already a bundled asset: "${assetPath}"`);
                    }
                    compilation.assets[assetPath] = { source: () => asset, size: () => asset.length };
                }
            }
        });
    }
    getDependency(config) {
        let dependency = this._dependencies.get(config.context);
        if (!dependency) {
            this._dependencies.set(config.context, dependency = { config, keys: new Set() });
        }
        return dependency;
    }
    bundleMessages(locale, messages, path) {
        let target = this._locales.get(locale);
        if (!target) {
            this._locales.set(locale, target = {});
        }
        messages_1.mergeMessages(target, messages_1.parseMessagePath(path || ""), messages);
    }
    getPackageConfig(context) {
        let request = this._packageConfigRequests.get(context);
        if (!request) {
            this._packageConfigRequests.set(context, request = config_1.tryLoadConfig(context));
        }
        return request;
    }
    async getPackageManifest(config) {
        let request = this._packageManifestRequests.get(config.context);
        if (!request) {
            this._packageManifestRequests.set(config.context, request = package_manifest_1.getPackageManifest(config));
        }
        return request;
    }
    processModule(compiler, module) {
        const config = module[common_1.I18N_MODULE_CONFIG_KEY];
        const meta = module[common_1.I18N_MODULE_META_KEY];
        if (meta && !this._processingModules.has(module.resource)) {
            const task = (async () => {
                const prefix = meta.getPrefix(config);
                const source = await this.readSource(compiler, module.resource);
                const entities = parse_source_1.parseSource(source);
                if (compiler.watchMode) {
                    const { pairs, output } = justify_source_1.justifySource(source, entities);
                    if (source !== output) {
                        await this.writeSource(module.resource, output);
                    }
                    this.files.set(module.resource, {
                        meta,
                        pairs: pairs.map(p => ({ key: prefix + p.key, value: p.value }))
                    });
                }
                else {
                    this.files.set(module.resource, {
                        meta,
                        pairs: entities
                            .filter(e => e.key !== undefined)
                            .map(e => ({ key: prefix + e.key, value: e.value }))
                    });
                }
            })();
            task.catch(() => { });
            this._processingModules.set(module.resource, task);
        }
    }
    readSource(compiler, filename) {
        let request = this._sourceRequests.get(filename);
        if (!request) {
            const fs = compiler.inputFileSystem;
            if (fs && fs.readFile) {
                request = new Promise((resolve, reject) => {
                    fs.readFile(filename, (error, source) => {
                        if (error) {
                            reject(error);
                        }
                        else {
                            resolve(typeof source === "string" ? source : source.toString("utf8"));
                        }
                    });
                });
            }
            else {
                request = fs_extra_1.readFile(filename, "utf8");
            }
        }
        return request;
    }
    async writeSource(filename, source) {
        this._sourceRequests.set(filename, Promise.resolve(source));
        await fs_extra_1.writeFile(filename, source);
    }
}
exports.I18nPlugin = I18nPlugin;
//# sourceMappingURL=i18n-plugin.js.map