"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const webpack = require("webpack");
const HtmlPlugin = require("html-webpack-plugin");
const vue_loader_1 = require("vue-loader");
const sassCompiler = require("sass");
const i18n_plugin_1 = require("./i18n-plugin");
const adapters_1 = require("../../i18n/adapters");
const CssExtractPlugin = require("mini-css-extract-plugin");
const i18nLoader = require.resolve("./i18n-loader");
async function run(config, context) {
    const i18nAdapter = await adapters_1.createI18nAdapter(config);
    const webpackConfig = {
        context: config.context,
        mode: context.env === "development" ? "development" : "production",
        devtool: context.env === "production" ? false : "inline-source-map",
        entry: "./src",
        resolve: {
            extensions: [".ts", ".js", ".json"],
            alias: {
                $vue: context.env === "development" ? "vue/dist/vue.runtime.js" : "vue/dist/vue.runtime.min.js"
            }
        },
        module: {
            rules: [
                { test: /\.js$/, loader: i18nLoader },
                { test: /\.ts$/, use: [
                        i18nLoader,
                        { loader: "ts-loader", options: {
                                appendTsSuffixTo: [/\.vue$/]
                            } }
                    ] },
                { test: /\.vue$/, use: [
                        i18nLoader,
                        { loader: "vue-loader", options: {
                                esModule: true,
                                productionMode: context.env === "production",
                                compilerOptions: {
                                    whitespace: "condense"
                                }
                            } }
                    ] },
                { test: /\.css$/, use: [
                        context.env === "production" ? CssExtractPlugin.loader : "vue-style-loader",
                        "css-loader"
                    ] },
                { test: /\.scss$/, use: [
                        context.env === "production" ? CssExtractPlugin.loader : "vue-style-loader",
                        "css-loader",
                        { loader: "sass-loader", options: {
                                implementation: sassCompiler
                            } }
                    ] }
            ]
        },
        plugins: [
            new i18n_plugin_1.I18nPlugin(config, context, i18nAdapter),
            new vue_loader_1.VueLoaderPlugin(),
            new HtmlPlugin({
                template: "./src/index.html",
                inject: "head",
                minify: {
                    collapseWhitespace: context.env === "production"
                }
            }),
            ...(context.env === "production" ? [
                new CssExtractPlugin({
                    filename: "bundle.[hash].css"
                })
            ] : [])
        ],
        node: false,
        output: {
            path: config.outDir,
            filename: "bundle.[hash].js",
            publicPath: "/"
        }
    };
    const compiler = webpack(webpackConfig);
    if (context.watch) {
        const Server = await Promise.resolve().then(() => require("webpack-dev-server"));
        const server = new Server(compiler, {
            hot: true,
            publicPath: "/"
        });
        server.listen(8080);
        await new Promise(() => { });
    }
    else {
        await new Promise((resolve, reject) => {
            compiler.run((error, stats) => {
                if (error) {
                    reject(error);
                }
                else {
                    console.log(stats.toString({
                        colors: process.stdout.isTTY
                    }));
                    if (stats.hasErrors()) {
                        process.exitCode = 1;
                    }
                    resolve();
                }
            });
        });
    }
}
exports.run = run;
//# sourceMappingURL=index.js.map