"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const vinyl_1 = require("../utility/vinyl");
const parse_source_1 = require("../../../i18n/parse-source");
const justify_source_1 = require("../../../i18n/justify-source");
const fs_extra_1 = require("fs-extra");
const file_meta_1 = require("../../../i18n/file-meta");
const path_1 = require("path");
function createI18nProcessor(context, justify) {
    const preprocessor = new stream_1.Transform({
        objectMode: true,
        async transform(chunk, encoding, callback) {
            try {
                await preprocess.call(this, chunk, context, justify);
                callback();
            }
            catch (error) {
                callback(error);
            }
        }
    });
    const postprocessor = new stream_1.Transform({
        objectMode: true,
        async transform(chunk, encoding, callback) {
            try {
                await postprocess.call(this, chunk, context);
                callback();
            }
            catch (error) {
                callback(error);
            }
        }
    });
    return { preprocessor, postprocessor };
}
exports.createI18nProcessor = createI18nProcessor;
async function preprocess(chunk, context, justify) {
    if (/\.(?:vue|ts|js)$/.test(chunk.path)) {
        const meta = file_meta_1.getFileMeta(chunk.path);
        if (meta) {
            const config = await context.getPackageConfig(meta.context);
            if (config) {
                const prefix = meta.getPrefix(config);
                const source = vinyl_1.getSource(chunk);
                const entities = parse_source_1.parseSource(source);
                if (justify) {
                    const { output, pairs } = justify_source_1.justifySource(source, entities);
                    if (source !== output) {
                        await fs_extra_1.writeFile(chunk.path, output);
                    }
                    context.files.set(chunk.path, {
                        meta,
                        pairs: pairs.map(p => ({ key: prefix + p.key, value: p.value }))
                    });
                }
                else {
                    context.files.set(chunk.path, {
                        meta,
                        pairs: entities
                            .filter(e => e.key !== undefined)
                            .map(e => ({ key: prefix + e.key, value: e.value }))
                    });
                }
            }
        }
    }
    this.push(chunk);
}
async function postprocess(chunk, context) {
    // Inject prefix into vue component entries:
    if (/\.vue\.js$/.test(chunk.path)) {
        const sourceFilename = chunk.history.find(p => /\.vue$/.test(p));
        if (sourceFilename) {
            const meta = file_meta_1.getFileMeta(sourceFilename);
            if (meta) {
                const config = await context.getPackageConfig(meta.context);
                if (config) {
                    context.setOutputSourceRelation(chunk.relative, sourceFilename);
                    const source = vinyl_1.getSource(chunk);
                    const prefix = meta.getPrefix(config);
                    chunk.contents = Buffer.from(injectVuePrefix(source, prefix));
                }
            }
        }
    }
    // Inject prefix into .ts/js files:
    // (gulp-typescript does not support vinyl history, so if the history has only one
    // entry, it is either an original .js file or a transpiled .ts file)
    if (/\.js$/.test(chunk.path) && chunk.history.length === 1) {
        let sourceFilename = chunk.path;
        const vueScriptSuffix = "--s.js";
        if (sourceFilename.endsWith(vueScriptSuffix)) {
            sourceFilename = sourceFilename.slice(0, -vueScriptSuffix.length) + ".js";
        }
        const meta = file_meta_1.getFileMeta(sourceFilename);
        if (meta) {
            const config = await context.getPackageConfig(meta.context);
            if (config) {
                if (sourceFilename.startsWith(config.outDir)) {
                    sourceFilename = path_1.join(config.rootDir, path_1.relative(config.outDir, sourceFilename));
                }
                context.setOutputSourceRelation(chunk.relative, sourceFilename);
                const source = vinyl_1.getSource(chunk);
                const prefix = meta.getPrefix(config);
                chunk.contents = Buffer.from(injectScriptPrefix(source, prefix));
            }
        }
    }
    this.push(chunk);
}
function injectVuePrefix(source, prefix) {
    return source.replace(`/* i18nPrefix */ null`, `/* i18nPrefix */ ${JSON.stringify(prefix)}`);
}
function injectScriptPrefix(source, prefix) {
    // Theoretically, the prefix constant should be injected after the first import and only if no import is called "i18nPrefix",
    // but with webpack as the final module bundler, it also works if the prefix is at the start of the file.
    return `const i18nPrefix = ${JSON.stringify(prefix)};\n${source}`;
}
//# sourceMappingURL=processor.js.map