"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const vinyl_fs_1 = require("vinyl-fs");
const chokidar = require("chokidar");
const gulpTs = require("gulp-typescript");
const gulpSass = require("gulp-sass");
const sourcemaps = require("gulp-sourcemaps");
const colors = require("ansi-colors");
const ts_1 = require("./utility/ts");
const task_1 = require("./utility/task");
const decompose_1 = require("./vue/decompose");
const error_1 = require("./vue/error");
const postprocess_1 = require("./postprocess");
const pipes_1 = require("./utility/pipes");
const error_2 = require("./sass/error");
const processor_1 = require("./i18n/processor");
const context_1 = require("./i18n/context");
const fs_extra_1 = require("fs-extra");
const package_manifest_1 = require("../../i18n/package-manifest");
const adapters_1 = require("../../i18n/adapters");
const messages_1 = require("../../i18n/messages");
async function run(config, context) {
    const i18nAdapter = await adapters_1.createI18nAdapter(config);
    const tsProject = gulpTs.createProject(path_1.join(config.context, "tsconfig.json"));
    const scheduler = new task_1.Task(async () => {
        let errorCount = 0;
        const i18nContext = new context_1.I18nContext(config, context);
        const input = vinyl_fs_1.src("./**/*", { cwd: config.rootDir, allowEmpty: true, nosort: true });
        const initSourcemaps = sourcemaps.init();
        const ts = tsProject({
            error(error) {
                const str = ts_1.formatTsError(error);
                if (str) {
                    errorCount++;
                    console.error("\n" + ts_1.formatTsError(error));
                }
            }
        });
        const scss = gulpSass({}).on("error", (error) => {
            errorCount++;
            console.error("\n" + error_2.formatSassError(error));
        });
        const vueDecomposer = decompose_1.createVueDecomposer({
            error(error) {
                errorCount++;
                console.error("\n" + error_1.formatVueError(error));
            }
        });
        const postprocessor = postprocess_1.createPostprocessor({
            error(error) {
                errorCount++;
                console.error("\n" + error_1.formatVueError(error));
            },
            vueDecomposer: vueDecomposer
        });
        const i18n = processor_1.createI18nProcessor(i18nContext, context.watch);
        const writeSourcemaps = sourcemaps.write();
        const output = vinyl_fs_1.dest(config.outDir);
        await new pipes_1.Pipes()
            .pipe(input, i18n.preprocessor)
            .pipe(i18n.preprocessor, initSourcemaps)
            .split(initSourcemaps, [
            (p, i) => p.route(i, [
                { map: /\.ts$/, to: ts },
                { map: /\.scss$/, to: scss },
                { map: /\.vue$/, to: vueDecomposer.stream },
                { to: postprocessor }
            ]),
            (p, i) => p.route(i, [
                { map: /\.scss$/, to: postprocessor }
            ])
        ])
            .route(vueDecomposer.stream, [
            { map: /\.d.ts$/, to: postprocessor },
            { map: /\.ts$/, to: ts },
            { map: /\.scss$/, to: scss },
            { to: postprocessor }
        ])
            .pipe(ts, postprocessor)
            .pipe(scss, postprocessor)
            .pipe(postprocessor, i18n.postprocessor)
            .pipe(i18n.postprocessor, writeSourcemaps)
            .pipe(writeSourcemaps, output)
            .run()
            .catch(error => {
            if (errorCount === 0) {
                throw error;
            }
        });
        if (i18nAdapter) {
            await i18nAdapter.process(i18nContext);
        }
        await fs_extra_1.ensureDir(path_1.join(config.outDir, "locale"));
        for (const [locale, messages] of i18nContext.locales) {
            await fs_extra_1.writeFile(path_1.join(config.outDir, messages_1.getLocaleMessageAssetPath(locale)), context.env === "production"
                ? JSON.stringify(messages)
                : JSON.stringify(messages, null, "\t"));
        }
        await fs_extra_1.writeFile(package_manifest_1.getPackageManifestFilename(config), JSON.stringify(i18nContext.generateManifest(), null, "\t"));
        if (errorCount > 0) {
            console.log(colors.redBright(`\n[${new Date().toLocaleTimeString()}] Compilation finished with ${errorCount} error(s).`));
        }
        else {
            console.log(colors.greenBright(`\n[${new Date().toLocaleTimeString()}] Compilation succeeded.`));
        }
        if (context.watch) {
            console.log(colors.gray("Watching for changes..."));
        }
        else if (errorCount > 0) {
            throw new Error("Build failed due to compilation errors.");
        }
    });
    if (context.watch) {
        console.clear();
        await scheduler.run();
        const watcher = chokidar.watch("./**/*", { cwd: config.rootDir, ignoreInitial: true });
        watcher.on("error", console.error);
        watcher.on("change", () => {
            console.clear();
            console.log(colors.gray("File change detected. Starting incremental compilation..."));
            scheduler.run();
        });
        await new Promise(() => { });
    }
    else {
        await scheduler.run();
    }
}
exports.run = run;
//# sourceMappingURL=index.js.map