"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
class Pipes {
    constructor() {
        this.outputs = new Map();
    }
    pipe(input, output) {
        const inputs = this.outputs.get(output);
        if (inputs) {
            inputs.add(input);
        }
        else {
            this.outputs.set(output, new Set([input]));
        }
        return this;
    }
    route(input, routes) {
        const routeOutputs = new Map();
        for (const route of routes) {
            routeOutputs.set(route, new stream_1.Readable({ objectMode: true, read() { } }));
        }
        const routeInput = new stream_1.Writable({
            objectMode: true,
            write(chunk, encoding, callback) {
                const route = routes.find(c => !c.map || c.map.test(chunk.path));
                if (route) {
                    routeOutputs.get(route).push(chunk);
                }
                callback();
            },
            final(callback) {
                for (const [, output] of routeOutputs) {
                    output.push(null);
                }
                callback();
            }
        });
        this.pipe(input, routeInput);
        for (const [route, output] of routeOutputs) {
            this.pipe(output, route.to);
        }
        return this;
    }
    split(input, handlers) {
        const splitOutputs = handlers.map(handler => {
            const output = new stream_1.Readable({ objectMode: true, read() { } });
            handler(this, output);
            return output;
        });
        const splitInput = new stream_1.Writable({
            objectMode: true,
            write(chunk, encoding, callback) {
                for (const output of splitOutputs) {
                    output.push(chunk.clone());
                }
                callback();
            },
            final(callback) {
                for (const output of splitOutputs) {
                    output.push(null);
                }
                callback();
            }
        });
        this.pipe(input, splitInput);
        return this;
    }
    run() {
        return new Promise((resolve, reject) => {
            const errors = [];
            // End an output, when all of it's inputs have ended:
            for (const [output, inputs] of this.outputs) {
                const pendingInputs = new Set(inputs);
                for (const input of inputs) {
                    input.on("end", () => {
                        if (pendingInputs.delete(input) && pendingInputs.size === 0) {
                            output.end();
                        }
                    });
                }
            }
            // Pipe all inputs to outputs:
            for (const [output, inputs] of this.outputs) {
                for (const input of inputs) {
                    input.on("error", error => {
                        errors.push(error);
                    });
                    input.pipe(output, { end: false });
                }
            }
            // Wait for all readable outputs to "end" and for all writable outputs to "finish":
            const pendingOutputs = new Set(this.outputs.keys());
            for (const output of this.outputs.keys()) {
                function done() {
                    if (pendingOutputs.delete(output) && pendingOutputs.size === 0) {
                        if (errors.length > 0) {
                            reject(errors[0]);
                        }
                        else {
                            resolve();
                        }
                    }
                }
                if (output.readable) {
                    output.on("end", done);
                    output.on("data", () => { });
                }
                else {
                    output.on("finish", done);
                }
            }
        });
    }
}
exports.Pipes = Pipes;
//# sourceMappingURL=pipes.js.map