"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const hash = require("hash-sum");
const stream_1 = require("stream");
const Vinyl = require("vinyl");
const vinyl_1 = require("../utility/vinyl");
const generator_1 = require("./generator");
const error_1 = require("./error");
/**
 * Create a transform stream that consumes vue single component files
 * and decomposes them into their individual parts.
 *
 * Emitted file types can be ".ts", ".js", ".scss" and ".css".
 * Before writing the final files to disk, they should be piped through
 * a composeVue transform to apply some final transformations.
 */
function createVueDecomposer(options) {
    const components = new Map();
    return {
        stream: new stream_1.Transform({
            objectMode: true,
            async transform(chunk, encoding, callback) {
                try {
                    await decompose.call(this, chunk, components);
                    callback();
                }
                catch (error) {
                    if (error instanceof error_1.VueError) {
                        options.error(error);
                        callback();
                    }
                    else {
                        callback(error);
                    }
                }
            }
        }),
        components
    };
}
exports.createVueDecomposer = createVueDecomposer;
async function decompose(chunk, components) {
    const name = chunk.path.slice(0, -chunk.extname.length);
    const source = vinyl_1.getSource(chunk);
    // Import compiler utils in demand to improve cli performance
    // for libraries that do not contain any vue components.
    const [vue, compiler] = await Promise.all([
        Promise.resolve().then(() => require("@vue/component-compiler-utils")),
        Promise.resolve().then(() => require("vue-template-compiler"))
    ]);
    const { template, script, styles } = vue.parse({
        source,
        compiler,
        filename: chunk.path,
        sourceRoot: path_1.dirname(chunk.path),
        needMap: true
    });
    const files = [];
    const scopeId = `data-v-${hash(`${chunk.relative}:${source}`)}`;
    const scoped = styles.some(s => s.scoped);
    let hasTemplate = false;
    if (template) {
        const { errors, code } = vue.compileTemplate({
            source: template.content,
            filename: chunk.path,
            compiler,
            compilerOptions: {
                outputSourceRange: true,
                whitespace: "condense"
            },
            transformAssetUrls: true,
            isProduction: true,
            isFunctional: false,
            optimizeSSR: false,
            prettify: true
        });
        if (errors.length > 0) {
            throw new error_1.VueError(chunk.path, errors);
        }
        files.push(new Vinyl({
            history: [...chunk.history],
            contents: Buffer.from(generator_1.templateModule({ code })),
            cwd: chunk.cwd,
            base: chunk.base,
            path: `${name}--r.js`
        }));
        hasTemplate = true;
    }
    let hasScript = false;
    let hasDeclaration = false;
    if (script) {
        if (script.lang !== "ts") {
            throw new error_1.VueError(chunk.path, [`Non typescript script blocks are currently unsupported in vue components. Use <script lang="ts">`]);
        }
        const scriptFile = new Vinyl({
            history: [...chunk.history],
            contents: Buffer.from(script.content),
            cwd: chunk.cwd,
            base: chunk.base,
            path: `${name}--s.ts`
        });
        // TODO: Attach source maps.
        files.push(scriptFile);
        hasScript = true;
        hasDeclaration = true;
    }
    let hasStyles = 0;
    const scopedStyles = new Set();
    for (let i = 0; i < styles.length; i++) {
        const style = styles[i];
        const styleFile = new Vinyl({
            history: [...chunk.history],
            contents: Buffer.from(style.content),
            cwd: chunk.cwd,
            base: chunk.base,
            path: `${name}--s${i}.${style.lang || "css"}`
        });
        if (style.scoped) {
            scopedStyles.add(i);
        }
        // TODO: Attach source maps.
        files.push(styleFile);
        hasStyles++;
    }
    files.push(new Vinyl({
        history: [...chunk.history],
        contents: Buffer.from(generator_1.componentEntry({ stem: chunk.stem, hasTemplate, hasScript, hasStyles, scoped, scopeId })),
        cwd: chunk.cwd,
        base: chunk.base,
        // The additional vue extension is used to allow imports of
        // the component without declarations for ".vue" files installed:
        path: `${name}.vue.js`
    }));
    files.push(new Vinyl({
        history: [...chunk.history],
        contents: Buffer.from(generator_1.componentDeclaration({ stem: chunk.stem, hasDeclaration })),
        cwd: chunk.cwd,
        base: chunk.base,
        path: `${name}.vue.d.ts`
    }));
    components.set(chunk.relative.slice(0, -chunk.extname.length), {
        scopeId,
        scopedStyles
    });
    for (const file of files) {
        this.push(file);
    }
}
//# sourceMappingURL=decompose.js.map